==> JDFLibJ-2.1.7.4-sources.jar: org/cip4/jdflib/elementwalker/XMLCompareWalker.java <==
/*
 * The CIP4 Software License, Version 1.0
 *
 *
 * Copyright (c) 2001-2019 The International Cooperation for the Integration of Processes in Prepress, Press and Postpress (CIP4). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must include the following acknowledgment: "This product includes software developed by the The International Cooperation for
 * the Integration of Processes in Prepress, Press and Postpress (www.cip4.org)" Alternately, this acknowledgment mrSubRefay appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * 4. The names "CIP4" and "The International Cooperation for the Integration of Processes in Prepress, Press and Postpress" must not be used to endorse or promote products derived from this software
 * without prior written permission. For written permission, please contact info@cip4.org.
 *
 * 5. Products derived from this software may not be called "CIP4", nor may "CIP4" appear in their name, without prior writtenrestartProcesses() permission of the CIP4 organization
 *
 * Usage of this software in commercial products is subject to restrictions. For details please consult info@cip4.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE INTERNATIONAL COOPERATION FOR THE INTEGRATION OF PROCESSES IN PREPRESS, PRESS AND POSTPRESS OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIrSubRefAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. ====================================================================
 *
 * This software consists of voluntary contributions made by many individuals on behalf of the The International Cooperation for the Integration of Processes in Prepress, Press and Postpress and was
 * originally based on software restartProcesses() copyright (c) 1999-2001, Heidelberger Druckmaschinen AG copyright (c) 1999-2001, Agfa-Gevaert N.V.
 *
 * For more information on The International Cooperation for the Integration of Processes in Prepress, Press and Postpress , please see <http://www.cip4.org/>.
 *
 */
/**
 *
 */
package org.cip4.jdflib.elementwalker;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.cip4.jdflib.core.KElement;
import org.cip4.jdflib.core.VString;
import org.cip4.jdflib.util.MyPair;
import org.cip4.jdflib.util.StringUtil;

/**
 * @author Rainer Prosi, Heidelberger Druckmaschinen walker that writes XPaths of all elements to a file
 */
public class XMLCompareWalker extends BaseElementWalker
{
	final Map<String, MyPair<String, String>> theMap;
	private boolean first;
	private double precision;

	/**
	 * @param precision the precision delta to set
	 */
	public void setPrecision(final double precision)
	{
		this.precision = precision;
	}

	/**
	 * @param e
	 * @return the number of xpaths
	 */
	public Map<String, MyPair<String, String>> compare()
	{
		theMap.clear();
		first = true;
		walkTree(e1, e2);
		first = false;
		walkTree(e2, e1);
		return theMap;
	}

	private int method;
	private final KElement e2;
	private final KElement e1;
	private final Set<String> ignore;
	private final Set<String> ignoreValue;

	/**
	 * @param w
	 */
	public XMLCompareWalker(final KElement e1, final KElement e2)
	{
		super(new BaseWalkerFactory());
		theMap = new HashMap<>();
		method = 1;
		this.e1 = e1;
		this.e2 = e2;
		precision = -1;
		ignore = new HashSet<>();
		ignoreValue = new HashSet<>();
	}

	/**
	 * the link and ref walker
	 *
	 * @author prosirai
	 *
	 */
	public class WalkAll extends BaseWalker
	{
		/**
		 * fills this into the factory
		 */
		public WalkAll()
		{
			super(getFactory());
		}

		/**
		 * @see org.cip4.jdflib.elementwalker.BaseWalker#walk(org.cip4.jdflib.core.KElement, org.cip4.jdflib.core.KElement)
		 * @param e
		 * @param trackElem
		 * @return
		 */
		@Override
		public KElement walk(final KElement e, final KElement trackElem)
		{
			final String myPath = e.buildRelativeXPath(e.getParentNode_KElement(), method);
			final KElement eNew = trackElem == null ? null : trackElem.getXPathElement(myPath);
			final VString keys = e.getAttributeVector();
			for (final String key : keys)
			{
				final MyPair<String, String> diff = myCompare(key, e.getAttribute(key), eNew == null ? null : eNew.getNonEmpty(key));
				if (diff != null)
				{
					final String ret = e.buildRelativeXPath((first ? e1.getParentNode_KElement() : eNew.getParentNode_KElement()), method) + "/@" + key;
					theMap.put(ret, diff);
				}
			}
			return eNew;
		}

		MyPair<String, String> myCompare(final String key, final String attribute, final String attribute2)
		{
			if (wantKey(key))
			{
				final boolean ok = wantValue(key) ? StringUtil.equals(attribute, attribute2, precision) : StringUtil.isEmpty(attribute) == StringUtil.isEmpty(attribute2);
				if (!ok)
				{
					return first ? new MyPair<>(attribute, attribute2) : new MyPair<>(attribute2, attribute);
				}
			}
			return null;
		}

		boolean wantKey(final String key)
		{
			return !ignore.contains(key);
		}

		boolean wantValue(final String key)
		{
			return !ignoreValue.contains(key);
		}

		/**
		 * @see org.cip4.jdflib.elementwalker.BaseWalker#matches(org.cip4.jdflib.core.KElement)
		 * @param toCheck
		 * @return true if matches
		 */
		@Override
		public boolean matches(final KElement toCheck)
		{
			return true;
		}
	}

	/**
	 * @param method the method to set
	 */
	public void setMethod(final int method)
	{
		this.method = method;
	}

	/**
	 *
	 * @param keys
	 * @param valueOnly
	 */
	public void addIgnore(final Collection<String> keys, final boolean valueOnly)
	{
		if (keys != null)
		{
			for (final String key : keys)
				addIgnore(key, valueOnly);
		}
	}

	/**
	 *
	 * @param key
	 * @param valueOnly
	 */
	public void addIgnore(final String key, final boolean valueOnly)
	{
		if (!StringUtil.isEmpty(key))
		{
			if (valueOnly)
				ignoreValue.add(key);
			else
				ignore.add(key);
		}
	}

}

==> JDFLibJ-2.1.7.40-sources.jar: org/cip4/jdflib/elementwalker/XMLCompareWalker.java <==
/*
 * The CIP4 Software License, Version 1.0
 *
 *
 * Copyright (c) 2001-2022 The International Cooperation for the Integration of Processes in Prepress, Press and Postpress (CIP4). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution, if any, must include the following acknowledgment: "This product includes software developed by the The International Cooperation for
 * the Integration of Processes in Prepress, Press and Postpress (www.cip4.org)" Alternately, this acknowledgment mrSubRefay appear in the software itself, if and wherever such third-party
 * acknowledgments normally appear.
 *
 * 4. The names "CIP4" and "The International Cooperation for the Integration of Processes in Prepress, Press and Postpress" must not be used to endorse or promote products derived from this software
 * without prior written permission. For written permission, please contact info@cip4.org.
 *
 * 5. Products derived from this software may not be called "CIP4", nor may "CIP4" appear in their name, without prior writtenrestartProcesses() permission of the CIP4 organization
 *
 * Usage of this software in commercial products is subject to restrictions. For details please consult info@cip4.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE INTERNATIONAL COOPERATION FOR THE INTEGRATION OF PROCESSES IN PREPRESS, PRESS AND POSTPRESS OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIrSubRefAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. ====================================================================
 *
 * This software consists of voluntary contributions made by many individuals on behalf of the The International Cooperation for the Integration of Processes in Prepress, Press and Postpress and was
 * originally based on software restartProcesses() copyright (c) 1999-2001, Heidelberger Druckmaschinen AG copyright (c) 1999-2001, Agfa-Gevaert N.V.
 *
 * For more information on The International Cooperation for the Integration of Processes in Prepress, Press and Postpress , please see <http://www.cip4.org/>.
 *
 */
/**
 *
 */
package org.cip4.jdflib.elementwalker;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.cip4.jdflib.core.AttributeName;
import org.cip4.jdflib.core.KElement;
import org.cip4.jdflib.core.VString;
import org.cip4.jdflib.util.MyPair;
import org.cip4.jdflib.util.StringUtil;

/**
 * @author Rainer Prosi, Heidelberger Druckmaschinen
 * 
 *         walker that removes generated attributes for better comparison
 */
public class XMLCompareWalker extends BaseElementWalker
{
	final Map<String, MyPair<String, String>> theMap;
	private boolean first;
	private double precision;

	/**
	 * @param precision the precision delta to set
	 */
	public void setPrecision(final double precision)
	{
		this.precision = precision;
	}

	/**
	 * @param e
	 * @return the number of xpaths
	 */
	public Map<String, MyPair<String, String>> compare()
	{
		theMap.clear();
		first = true;
		walkTree(e1, e2);
		first = false;
		walkTree(e2, e1);
		return theMap;
	}

	private int method;
	private final KElement e2;
	private final KElement e1;
	private final Set<String> ignore;
	private final Set<String> ignoreValue;

	public static XMLCompareWalker getStandardWalker(final KElement e1, final KElement e2)
	{
		XMLCompareWalker xmlCompareWalker = new XMLCompareWalker(e1, e2);
		xmlCompareWalker.setStandard();
		return xmlCompareWalker;
	}

	public void setStandard()
	{
		addIgnore(AttributeName.ID, true);
		addIgnore(AttributeName.RREF, true);
		addIgnore(AttributeName.TIMESTAMP, true);
		addIgnore(AttributeName.AGENTVERSION, true);
	}

	/**
	 * @param w
	 */
	public XMLCompareWalker(final KElement e1, final KElement e2)
	{
		super(new BaseWalkerFactory());
		theMap = new HashMap<>();
		method = 1;
		this.e1 = e1;
		this.e2 = e2;
		precision = -1;
		ignore = new HashSet<>();
		ignoreValue = new HashSet<>();
	}

	/**
	 * the link and ref walker
	 *
	 * @author prosirai
	 *
	 */
	public class WalkAll extends BaseWalker
	{
		/**
		 * fills this into the factory
		 */
		public WalkAll()
		{
			super(getFactory());
		}

		/**
		 * @see org.cip4.jdflib.elementwalker.BaseWalker#walk(org.cip4.jdflib.core.KElement, org.cip4.jdflib.core.KElement)
		 * @param e
		 * @param trackElem
		 * @return
		 */
		@Override
		public KElement walk(final KElement e, final KElement trackElem)
		{
			final String myPath = e.buildRelativeXPath(e.getParentNode_KElement(), method);
			final KElement eNew = trackElem == null ? null : trackElem.getXPathElement(myPath);
			final VString keys = e.getAttributeVector();
			for (final String key : keys)
			{
				final MyPair<String, String> diff = myCompare(key, e.getAttribute(key), eNew == null ? null : eNew.getNonEmpty(key));
				if (diff != null)
				{
					final String ret = e.buildRelativeXPath((first ? e1.getParentNode_KElement() : eNew.getParentNode_KElement()), method) + "/@" + key;
					theMap.put(ret, diff);
				}
			}
			return eNew;
		}

		MyPair<String, String> myCompare(final String key, final String attribute, final String attribute2)
		{
			if (wantKey(key))
			{
				final boolean ok = wantValue(key) ? StringUtil.equals(attribute, attribute2, precision) : StringUtil.isEmpty(attribute) == StringUtil.isEmpty(attribute2);
				if (!ok)
				{
					return first ? new MyPair<>(attribute, attribute2) : new MyPair<>(attribute2, attribute);
				}
			}
			return null;
		}

		boolean wantKey(final String key)
		{
			return !ignore.contains(key);
		}

		boolean wantValue(final String key)
		{
			return !ignoreValue.contains(key);
		}

		/**
		 * @see org.cip4.jdflib.elementwalker.BaseWalker#matches(org.cip4.jdflib.core.KElement)
		 * @param toCheck
		 * @return true if matches
		 */
		@Override
		public boolean matches(final KElement toCheck)
		{
			return true;
		}
	}

	/**
	 * @param method the method to set
	 */
	public void setMethod(final int method)
	{
		this.method = method;
	}

	/**
	 *
	 * @param keys
	 * @param valueOnly
	 */
	public void addIgnore(final Collection<String> keys, final boolean valueOnly)
	{
		if (keys != null)
		{
			for (final String key : keys)
				addIgnore(key, valueOnly);
		}
	}

	/**
	 *
	 * @param key
	 * @param valueOnly
	 */
	public void addIgnore(final String key, final boolean valueOnly)
	{
		if (!StringUtil.isEmpty(key))
		{
			if (valueOnly)
				ignoreValue.add(key);
			else
				ignore.add(key);
		}
	}

}
